package net.beaumaris.cachetest;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Random;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.Executors;
import java.util.concurrent.ThreadPoolExecutor;

import junit.framework.Assert;

import org.apache.log4j.Logger;
import org.infinispan.manager.DefaultCacheManager;
import org.infinispan.manager.EmbeddedCacheManager;

public abstract class BaseCacheTest {

   private static Logger log = Logger.getLogger(BaseCacheTest.class.getName());

   protected final static int NUM_NODES = 3;
   protected final static int NUM_THREADS = 2;
   protected final static int COUNT = 100;

   protected final static int KEY_UPPER_BOUND = 10;
   protected final static int INCREMENT_AMOUNT = 1000;

   protected static Random r = new Random();

   protected List<DefaultCacheManager> cacheContainers = Collections.synchronizedList(new ArrayList<DefaultCacheManager>(NUM_NODES));

   public void createNodes() throws InterruptedException, IOException {

      for (int nodeNum = 0; nodeNum < NUM_NODES; nodeNum++) {
         addNode();
      }
   }

   public void shutdown() {

      for (DefaultCacheManager cm : cacheContainers) {

         if (cm != null)
            cm.stop();
      }

      cacheContainers.clear();
   }

   protected void addNode() throws IOException {

      log.info("Adding a node...");
      DefaultCacheManager cm = new DefaultCacheManager("cluster.xml");
      cm.start();
      cacheContainers.add(cm);
   }

   protected void removeNode() throws IOException {

      log.info("Removing a node...");
      int size = cacheContainers.size();
      int index = r.nextInt(size);
      EmbeddedCacheManager cacheManager = cacheContainers.remove(index);
      cacheManager.stop();
   }

   protected void startThreads(final CountDownLatch latch, SharedStats stats, ThreadPoolExecutor executor) {

      for (DefaultCacheManager cm : cacheContainers) {

         for (int i = 0; i < NUM_THREADS; i++) {

            BaseStressingThread thread = createStressingThread(stats, cm, latch);
            executor.execute(thread);
         }
      }
   }

   protected void doRehashJoin() throws Exception {

      CountDownLatch latch = new CountDownLatch(NUM_NODES * NUM_THREADS);
      SharedStats stats = new SharedStats();
      ThreadPoolExecutor executor = (ThreadPoolExecutor) Executors.newFixedThreadPool(NUM_NODES * NUM_THREADS);

      startThreads(latch, stats, executor);

      // Wait for a few seconds.
      Thread.sleep(15000);

      // Try and add a cache
      try {
         addNode();
      } catch (Exception e) {
         Assert.fail();
      }

      // Try and add a cache
      try {
         addNode();
      } catch (Exception e) {
         Assert.fail();
      }

      latch.await();
      executor.shutdown();

      log.info(stats);
   }

   protected abstract BaseStressingThread createStressingThread(SharedStats stats, DefaultCacheManager cm, CountDownLatch latch);
}
