package net.beaumaris.cachetest;

import java.util.concurrent.CountDownLatch;

import org.infinispan.distexec.DistributedExecutorService;
import org.infinispan.manager.DefaultCacheManager;

public abstract class BaseStressingThread implements Runnable {

   protected static final long SLEEP_MILLISECONDS = 10;

   protected final SharedStats stats;
   protected final CountDownLatch latch;
   protected final int count;

   protected DistributedExecutorService executor;

   public BaseStressingThread(SharedStats stats, DefaultCacheManager cacheManager, final CountDownLatch latch, int count) {

      this.stats = stats;
      this.latch = latch;
      this.count = count;

      initExecutor(cacheManager);
   }

   public void run() {

      int i = 0;
      int errorCount = 0;

      while (i < count && !(stats.globalQuit || Thread.interrupted())) {

         try {
            doCycle();
         } catch (Exception e) {

            errorCount++;
            checkIsTrue(false, e.getMessage());
         }

         i++;
      }

      latch.countDown();
   }

   protected abstract void doCycle() throws Exception;

   protected abstract void initExecutor(DefaultCacheManager cacheManager);

   protected void sleep(long milliseconds) {
      try {
         Thread.sleep(milliseconds);
      } catch (InterruptedException e) {
         // no-op: waking up is good enough
      }
   }

   protected void checkIsTrue(boolean assertion, String message) {
      if (assertion == false) {
         stats.seenFailures = true;
         if (message != null)
            System.out.println(message);
      }
   }
}
