package net.beaumaris.cachetest;

import java.util.Set;

import javax.transaction.Status;
import javax.transaction.SystemException;
import javax.transaction.TransactionManager;

import org.infinispan.Cache;
import org.infinispan.distexec.DistributedCallable;

public abstract class BaseTask<K, T, A, R> implements DistributedCallable<K, A, T> {

   private static final long serialVersionUID = 1L;

   protected Cache<K, A> allocCache;
   protected Cache<String, R> uuidCache;

   private TransactionManager tm;

   @Override
   public T call() throws Exception {
      try {

         tm.begin();

         return doTx();

      } catch (Exception e) {

         tm.setRollbackOnly();
         throw e;
      }

      finally {

         int status = -1;
         try {
            status = tm.getStatus();
         } catch (Exception e) {

         }

         if (status == Status.STATUS_ACTIVE) {
            tm.commit();
         } else {
            tm.rollback();
         }
      }
   }

   protected abstract T doTx() throws Exception;

   protected void markForRollback() throws IllegalStateException, SystemException {
      if (tm != null)
         tm.setRollbackOnly();
   }

   @Override
   public void setEnvironment(Cache<K, A> cache, Set<K> inputKeys) {

      this.allocCache = cache;
      this.uuidCache = cache.getCacheManager().getCache("uuid");
      this.tm = cache.getAdvancedCache().getTransactionManager();
   }
}
