package net.beaumaris.cachetest;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.infinispan.Cache;
import org.infinispan.remoting.transport.Address;

import com.thoughtworks.xstream.XStream;

public class PrintUtils {

   private static XStream xstream = new XStream();
   
   public static void getCacheContent(final Cache cache, int index) throws IOException {

      Set<Map.Entry> entrySet = cache.entrySet();

      File outputFile = new File(".", "cacheManager" + index + "_" + cache.getName() + ".xml");
      ObjectOutputStream out = xstream.createObjectOutputStream(new BufferedOutputStream(new FileOutputStream(outputFile)));
      try {
         out.writeObject(entrySet.size());
         for (Map.Entry entry : entrySet) {

            if (isLocal(cache, entry.getKey())) {

               out.writeObject(entry.getKey());
               out.writeObject(entry.getValue());
            }
         }
      } finally {
         out.close();
      }
   }

   private static boolean isLocal(Cache<?, ?> cache, Object key) {
      Address self = cache.getAdvancedCache().getRpcManager().getAddress();

      // This will ask the consistent hash function for the data owners.  This is sorted by the data owners
      // with the "main" one at the head of the list.
      if (cache.getAdvancedCache().getDistributionManager() == null) {
         return true;
      }

      List<Address> locations = cache.getAdvancedCache().getDistributionManager().locate(key);
      if (locations != null && !locations.isEmpty()) {
         return (self.equals(locations.get(0)));
      }
      return false;
   }

   private PrintUtils() {
   }
}
