package net.beaumaris.cachetest;

import java.io.IOException;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;

import javax.transaction.TransactionManager;

import org.infinispan.Cache;
import org.infinispan.distexec.DefaultExecutorService;
import org.infinispan.manager.DefaultCacheManager;
import org.junit.After;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

public class RehashTest extends BaseCacheTest {

   @BeforeClass
   public static void init() {
      System.setProperty("java.net.preferIPv4Stack", "true");
   }

   @Before
   public void setup() throws InterruptedException, IOException {
      this.createNodes();
   }

   @After
   public void stop() throws Exception {

      // Try and acquire all locks
      for (int i = 1; i < KEY_UPPER_BOUND; i++) {

         int index = r.nextInt(cacheContainers.size());
         DefaultCacheManager cm = cacheContainers.get(index);
         Cache<Integer, Integer> allocCache = cm.getCache("alloc");

         TransactionManager tm = allocCache.getAdvancedCache().getTransactionManager();

         try {

            tm.begin();

            allocCache.getAdvancedCache().lock(i);

            Integer value = allocCache.get(i);
            if (value == null)
               value = new Integer(0);

            allocCache.put(i, value);
            tm.commit();

         } catch (Exception e) {
            tm.rollback();
            throw e;
         }
      }

      this.shutdown();
   }

   @Test
   public void testRehashJoin() throws Exception {
      this.doRehashJoin();
   }

   @Override
   protected BaseStressingThread createStressingThread(SharedStats stats, DefaultCacheManager cm, CountDownLatch latch) {
      return new AdditionStressingThread(stats, cm, latch);
   }

   public class AdditionStressingThread extends BaseStressingThread {

      public AdditionStressingThread(SharedStats stats, DefaultCacheManager cacheManager, final CountDownLatch latch) {
         super(stats, cacheManager, latch, COUNT);
      }

      @Override
      protected void doCycle() throws Exception {

         // Pick a key.
         int key = r.nextInt(KEY_UPPER_BOUND) + 1;

         AddTask addTask = new AddTask(key, INCREMENT_AMOUNT);
         Future<String> addResult = executor.submit(addTask, key);

         stats.allocateSuccessCounter.getAndIncrement();

         String uuid = addResult.get(2000, TimeUnit.MILLISECONDS);

         sleep(1000);

         SubtractTask subtractTask = new SubtractTask(uuid, INCREMENT_AMOUNT);

         Future<Integer> subtractResult = executor.submit(subtractTask, key);
         Integer result = subtractResult.get(2000, TimeUnit.MILLISECONDS);

         if (result == 0)
            stats.deallocateSuccessCounter.getAndIncrement();
         else
            stats.deallocateFailedCounter.getAndIncrement();

         // Wait before retrying.
         sleep(1000);
      }

      @Override
      protected void initExecutor(DefaultCacheManager cacheManager) {

         Cache<Integer, Integer> allocCache = cacheManager.getCache("alloc");
         executor = new DefaultExecutorService(allocCache);
      }
   }
}
