package net.beaumaris.cachetest.group;

import java.io.Serializable;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import net.beaumaris.cachetest.group.MockResource.ResourceState;

public class AllocResource implements Serializable {

   /**
    * Default UID.
    */
   private static final long serialVersionUID = 1L;

   private int current = 0;
   private final List<MockResource> resourceList;

   public AllocResource(int numResources) {
      resourceList = new ArrayList<MockResource>(numResources);
      for (int i = 0; i < numResources; i++) {
         resourceList.add(new MockResource());
      }
   }

   public void assignResource(int amount, int index, String uuid) {

      MockResource m = getResource(index);
      m.assignSession(uuid);
      this.current += amount;
   }

   public void releaseResource(int amount, int index, String uuid) {

      MockResource m = getResource(index);
      m.removeSession(uuid);
      this.current -= amount;
   }

   public int findResource() {

      for (int i = 0; i < resourceList.size(); i++) {

         MockResource m = resourceList.get(i);
         if (m.getResourceState() == ResourceState.UNUSED)
            return i;
      }

      return -1;
   }

   private MockResource getResource(int index) {

      if (index < 0 || index >= resourceList.size())
         throw new RuntimeException(MessageFormat.format("AllocResource::assignResource. Index: {0}. Resource size: {1}", index, resourceList.size()));

      MockResource m = resourceList.get(index);
      return m;
   }

   /**
    * @return the current
    */
   public int getCurrent() {
      return current;
   }

   /**
    * @param current
    *           the current to set
    */
   public void setCurrent(int current) {
      this.current = current;
   }

   /**
    * @return the resourceList
    */
   public List<MockResource> getResourceList() {
      return Collections.unmodifiableList(resourceList);
   }
}
