package net.beaumaris.cachetest.group;

import java.io.Serializable;
import java.util.List;
import java.util.Random;
import java.util.UUID;

import net.beaumaris.cachetest.BaseTask;

import org.apache.log4j.Logger;
import org.infinispan.context.Flag;

public class AllocateTask extends BaseTask<GroupedKey, String, AllocResource, Session> implements Serializable {

   /**
    * Default UID.
    */
   private static final long serialVersionUID = 1L;

   private static Logger log = Logger.getLogger(AllocateTask.class.getName());

   private final List<GroupedKey> keys;
   private final Random r = new Random();
   private final int numResources;

   public AllocateTask(List<GroupedKey> keys, int numResources) {
      this.keys = keys;
      this.numResources = numResources;
   }

   @Override
   protected String doTx() throws Exception {

      // Pick a random key and lock it.
      int index = r.nextInt(keys.size());
      GroupedKey selectedKey = keys.get(index);

      allocCache.getAdvancedCache().lock(selectedKey);

      int amount = 1; // For the hell of it.

      String uuid = UUID.randomUUID().toString();

      AllocResource value = allocCache.get(selectedKey);
      if (value == null) {
         value = new AllocResource(numResources);
      }

      int resourceIndex = value.findResource();
      if (resourceIndex == -1) {
         log.warn("AllocateTask::doTx(). No resource found for key " + selectedKey);
         this.markForRollback();
         return null;
      }

      value.assignResource(amount, resourceIndex, uuid);
      allocCache.put(selectedKey, value);

      Session resource = new Session(selectedKey, amount, uuid, resourceIndex);
      this.uuidCache.getAdvancedCache().withFlags(Flag.SKIP_REMOTE_LOOKUP).put(uuid, resource);

      return uuid;
   }
}
