package net.beaumaris.cachetest.group;

import java.io.Serializable;

public class GroupedKey implements Serializable, Comparable<GroupedKey> {

   /**
    * Default UID
    */
   private static final long serialVersionUID = 1L;

   private final int resourceId;
   private final int groupId;

   public GroupedKey(int resourceId, int groupId) {
      this.resourceId = resourceId;
      this.groupId = groupId;
   }

   /**
    * @return the resourceId
    */
   public int getResourceId() {
      return resourceId;
   }

   /**
    * @return the groupId
    */
   //@Group
   public String getGroupId() {
      return "GROUP" + groupId;
   }

   /* (non-Javadoc)
    * @see java.lang.Object#hashCode()
    */
   @Override
   public int hashCode() {
      /*
      final int prime = 31;
      int result = 1;
      result = prime * result + groupId;
      result = prime * result + resourceId;
      return result;
      */
      return groupId;
   }

   /* (non-Javadoc)
    * @see java.lang.Object#equals(java.lang.Object)
    */
   @Override
   public boolean equals(Object obj) {
      if (this == obj) {
         return true;
      }
      if (obj == null) {
         return false;
      }
      if (!(obj instanceof GroupedKey)) {
         return false;
      }
      GroupedKey other = (GroupedKey) obj;
      if (groupId != other.groupId) {
         return false;
      }
      if (resourceId != other.resourceId) {
         return false;
      }
      return true;
   }

   @Override
   public int compareTo(GroupedKey o) {

      final int BEFORE = -1;
      final int EQUAL = 0;
      final int AFTER = 1;

      if (this.equals(o))
         return EQUAL;

      else if (this.groupId > o.groupId)
         return BEFORE;

      else if (this.groupId < o.groupId)
         return AFTER;
      else {

         if (this.resourceId > o.resourceId)
            return BEFORE;
         else
            return AFTER;
      }
   }

   /*
    * (non-Javadoc)
    * @see java.lang.Object#toString()
    */
   @Override
   public String toString() {
      return "GroupedKey [resourceId=" + resourceId + ", groupId=" + groupId + "]";
   }
}
